@@ -5,12 +5,16 @@ import pyhcup
 
 from django.utils.timezone import utc
 from django.db import models
+from django.db.models import Q
 from django.utils.html import escape
 from django.core.urlresolvers import reverse
 from django.conf import settings
 
 from djhcup_core.defaults import DATETIME_FORMAT
 
+# set some constants
+ALLOWED_LOG_LEVELS = ['NOTSET', 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']
+
 
 # start a logger
 default_logger = logging.getLogger('default')
@@ -233,13 +237,13 @@ class ImportBatch(models.Model):
     
     def __unicode__(self):
         c_items = ImportQueue.objects.filter(batch=self).count()
-        return "Batch %i [%s]: %d files" % (self.pk, self.status, c_items)
+        return "ImportBatch %i [%s]: %d files" % (self.pk, self.status, c_items)
 
 
 class ImportQueue(models.Model):
     batch = models.ForeignKey('ImportBatch')
     file = models.ForeignKey('File',
-        limit_choices_to = {'loadfile__isnull': False,},
+        limit_choices_to = Q(loadfile__isnull=False, file_type__content_type='CONTENTNH') | Q(file_type__content_type='CONTENTWH'),
         help_text="Must point to a non-loadfile with a loadfile matched to it; this file and the import batch's destination must belong to the same data source or the batch will fail to execute")
     start = models.DateTimeField(blank=True, null=True, default=None)
     complete = models.DateTimeField(blank=True, null=True, default=None)
@@ -312,15 +316,15 @@ class ImportQueue(models.Model):
     
     
     class Meta:
-        verbose_name = 'enqueued item'
-        verbose_name_plural = 'enqueued items'
+        verbose_name = 'enqueued import'
+        verbose_name_plural = 'enqueued imports'
     
     
     def log(self, message, level='INFO', status=None, logger=None):
         """Emit a log message at the specified log level and optionally update the ImportQueue object status attribute.
         """
         
-        if level not in ['NOTSET', 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']:
+        if level not in ALLOWED_LOG_LEVELS:
             raise Exception("Invalid log level specified: %s" % level)
         else:
             level = getattr(logging, level)
@@ -334,7 +338,8 @@ class ImportQueue(models.Model):
         self.message = message
         self.save()
         
-        logger.log(level, message)
+        logger_message = "[ImportQueue item %s] %s" % (self.pk, message)
+        logger.log(level, logger_message)
         return True
     
     
@@ -435,30 +440,286 @@ class ImportQueue(models.Model):
     
     
     def __unicode__(self):
-        return "Enqueued item %s [%s]: %s" % (self.pk, self.status, self.file.filename)
+        return "ImportQueue item %s [%s]: %s" % (self.pk, self.status, self.file.filename)
+
+
+class StagingBatch(models.Model):
+    #destination = models.ForeignKey('ImportTable')
+    name = models.CharField(max_length=200, blank=True, help_text="Optional name for this batch of staging work")
+    description = models.TextField(blank=True, help_text="Optional description for this batch of staging work")
+    STATUS_CHOICES = [('NEW', 'Not started'),
+                      ('QUEUED', 'Request for processing sent'),
+                      ('DISPATCHED', 'Sent to Celery'),
+                      ('RECEIVED', 'Received by Celery'),
+                      ('IN PROCESS', 'Running'),
+                      ('FAILED', 'Failed to complete'),
+                      ('SUCCESS', 'Completed successfully'),
+                      ]
+    status = models.CharField(max_length=15, blank=False, null=True, default='NEW', choices=STATUS_CHOICES)
+    celery_task_id = models.TextField(blank=True, null=True, default=None)
+    
+    start = models.DateTimeField(blank=True, null=True, default=None)
+    complete = models.DateTimeField(blank=True, null=True, default=None)
+
+    #meta cruft
+    created_at = models.DateTimeField(auto_now_add=True)
+    #created_by = models.ForeignKey(django user object) #unimplemented
+    void = models.BooleanField(default=False)
+    voided_at = models.DateTimeField(blank=True, null=True, default=None)
+    #voided_by = models.ForeignKey(django user object) #unimplemented
+    
+    class Meta:
+        verbose_name_plural = 'staging batches'
+        
+    def _hr_start(self):
+        return self.start.strftime(DATETIME_FORMAT)
+    hr_start = property(_hr_start)
+    
+    def _hr_complete(self):
+        return self.complete.strftime(DATETIME_FORMAT)
+    hr_complete = property(_hr_complete)
+    
+    def _runtime(self):
+        if self.start and self.complete:
+            return self.complete - self.start
+        else:
+            return None
+    runtime = property(_runtime)
+    
+    def _hr_runtime(self):
+        """Batch runtime in hours"""
+        if self.runtime is not None:
+            td = self.runtime
+            return td.days*24+td.seconds/float(3600)
+        else:
+            return None
+    hr_runtime = property(_hr_runtime)
+    
+    
+    def fail(self):
+        self.status = 'FAILED'
+        self.complete = datetime.datetime.utcnow().replace(tzinfo=utc)
+        self.save()
+    
+    
+    def success(self):
+        self.status = 'SUCCESS'
+        self.complete = datetime.datetime.utcnow().replace(tzinfo=utc)
+        self.save()
+    
+    
+    def successful(self):
+        if self.status == 'SUCCESS':
+            return True
+        else:
+            return False
+    
+    
+    def get_absolute_url(self):
+        return reverse('djhcup_staging.views.stagingbatch_detail', kwargs={'batch_id': self.pk})
+    
+    def __unicode__(self):
+        c_items = StagingQueue.objects.filter(batch=self).count()
+        return "StagingBatch %i [%s]: %d files" % (self.pk, self.status, c_items)
+
+
+class StagingQueue(models.Model):
+    batch = models.ForeignKey('StagingBatch')
+    input_ht = '''
+        In the case of staging operations with a single input, set
+        this attribute equal to it. For staging operations with
+        multiple inputs, this is typically a reference to a CORE_SOURCE table.
+        '''
+    stg_input = models.ForeignKey('StagingTable', related_name='stg_input',
+                                  help_text=input_ht)
+    charges_input = models.ForeignKey('StagingTable', related_name='charges_input',
+                                      help_text='CHARGES, if required.',
+                                      null=True, default=None, blank=True)
+    pr_input = models.ForeignKey('StagingTable', related_name='pr_input',
+                                 help_text='PR, if required.',
+                                 null=True, default=None, blank=True)
+    uflag_input = models.ForeignKey('StagingTable', related_name='uflag_input',
+                                    help_text='UFLAGS, if required.',
+                                    null=True, default=None, blank=True)
+    dte_input = models.ForeignKey('StagingTable', related_name='dte_input',
+                                  help_text='DTE_SOURCE, if required.',
+                                  null=True, default=None, blank=True)
+    aha_input = models.ForeignKey('StagingTable', related_name='aha_input',
+                                  help_text='AHA_SOURCE, if required.',
+                                  null=True, default=None, blank=True)
+    
+    
+    # Anyone looking at this code must, by now, wonder why more of
+    # this wasn't made in a modular fashion. For example, why not
+    # have categories of table types be held in their own table,
+    # so that new categories can be introduced and logic for
+    # managing their transformations could be generated
+    # dynamically?
+    #
+    # The answer is: that's total overkill. We have fewer than
+    # ten distinct transformations, even if you include the
+    # initial loading (which I don't, as it is semantically quite
+    # different). And some kind of generalizable code for column
+    # A becomes column B with transformation C in between is way
+    # more trouble than it is worth.
+    #
+    # Similarly, making all the "Queue" style objects into a
+    # single model, where some are file imports off a disk and
+    # others are transformations of data in staging tables, is
+    # perhaps the height of idealism but unquestionably the utter
+    # dregs of pedantry. At this scale, it is an incredibly
+    # inefficient approach. And, at a tremendously increased
+    # scale, the overhead of a second table due to dividing work
+    # into within-database and into-database is dwarfed by the
+    # weight of everything else.
+    #
+    # Another question surely brought to mind by now: why this
+    # missive?
+    #
+    # The answer is: in case I start thinking consolidating these
+    # might be a good idea.
+    
+    CAT_CHOICES = [
+        #('CORE_SOURCE','CORE data in wide format, from source (no derived/updated cols)'),
+        #('CHARGES_SOURCE','Charges data, from source (no wide-to-long conversion)'),
+        #('DAYSTOEVENT_SOURCE','DaysToEvent and VisitLink data, from source (straight csv load)'),
+        #('AHALINK_SOURCE','American Hospital Association link data, from source (straight csv load)'),
+        ('CHARGES','Charges data in long format (converted from wide source if necessary)'),
+        ('DX','Diagnoses data in long format (from CORE_SOURCE)'),
+        ('PR','Procedures data in long format (from CORE_SOURCE)'),
+        ('UFLAGS','Utilization flag data, long format (from CORE_SOURCE, CHARGES, and PR)'),
+        ('CORE_PROCESSED','CORE Processed: CORE, DaysToEvent/VisitLink, wide charges, wide uflags'),
+    ]
+    category = models.CharField(max_length=100, choices=CAT_CHOICES, blank=False, help_text="Category of data to build from stg_input and insert into stg_output")
+    
+    stg_output = models.ForeignKey('StagingTable', related_name='stg_output', default=None, blank=True, null=True)
+    start = models.DateTimeField(blank=True, null=True, default=None)
+    complete = models.DateTimeField(blank=True, null=True, default=None)
+    STATUS_CHOICES = [('NEW', 'Not started'),
+                      ('QUEUED', 'Request for processing sent'),
+                      ('DISPATCHED', 'Sent to Celery'),
+                      ('RECEIVED', 'Received by Celery'),
+                      ('IN PROCESS', 'Running'),
+                      ('FAILED', 'Failed to complete'),
+                      ('SUCCESS', 'Completed successfully'),
+                      ]
+    status = models.CharField(max_length=15, blank=False, null=True, default='NEW', choices=STATUS_CHOICES)
+    message = models.TextField(blank=True, null=True, default=None)
+    message_at = models.DateTimeField(blank=True, null=True, default=None)
+    celery_task_id = models.TextField(blank=True, null=True, default=None)
+    
+    #meta cruft
+    created_at = models.DateTimeField(auto_now_add=True)
+    #created_by = models.ForeignKey(django user object) #unimplemented
+    void = models.BooleanField(default=False)
+    voided_at = models.DateTimeField(blank=True, null=True, default=None)
+    #voided_by = models.ForeignKey(django user object) #unimplemented
+    
+    def _hr_start(self):
+        return self.start.strftime(DATETIME_FORMAT)
+    hr_start = property(_hr_start)
+    
+    def _hr_complete(self):
+        return self.complete.strftime(DATETIME_FORMAT)
+    hr_complete = property(_hr_complete)
+    
+    def _runtime(self):
+        if self.start and self.complete:
+            return self.complete - self.start
+        else:
+            return None
+    runtime = property(_runtime)
+    
+    def _hr_runtime(self):
+        """File runtime in hours"""
+        if self.runtime is not None:
+            td = self.runtime
+            return td.days*24+td.seconds/float(3600)
+        else:
+            return None
+    hr_runtime = property(_hr_runtime)
+    
+    class Meta:
+        verbose_name = 'enqueued staging'
+        verbose_name_plural = 'enqueued stagings'
+    
+    def log(self, message, level='INFO', status=None, logger=None):
+        """Emit a log message at the specified log level and optionally update the ImportQueue object status attribute.
+        """
+        
+        if level not in ALLOWED_LOG_LEVELS:
+            raise Exception("Invalid log level specified: %s" % level)
+        else:
+            level = getattr(logging, level)
+        
+        if logger is None:
+            logger = default_logger
+        
+        if status is not None:
+            self.status = status
+        
+        self.message = message
+        self.save()
+        
+        logger_message = "[StagingQueue item %s] %s" % (self.pk, message)
+        
+        logger.log(level, logger_message)
+        return True
+    
+    def fail(self, message=None, logger=None, level='ERROR'):
+        self.status = 'FAILED'
+        self.complete = datetime.datetime.utcnow().replace(tzinfo=utc)
+        self.save()
+        
+        if message is not None:
+            self.log(message, level=level, logger=logger)
+        
+        return True
+    
+    def success(self, message=None, logger=None):
+        self.status = 'SUCCESS'
+        self.complete = datetime.datetime.utcnow().replace(tzinfo=utc)
+        self.save()
+        
+        if message is not None:
+            self.log(message, logger=logger)
+        
+        return True
+    
+    def successful(self):
+        if self.status == 'SUCCESS':
+            return True
+        else:
+            return False
+    
+    def __unicode__(self):
+        return "StagingQueue item %s [%s]: %s" % (self.pk, self.status, self.stg_input)
 
 
 class StagingTable(models.Model):
     """This is a record of staging tables created/populated
     """
     #source = models.ForeignKey('DataSource', help_text="Data source for which this table was created as a 'master' import table")
-    imported_via = models.ForeignKey('ImportQueue', blank=True, default=None)
-    #parent = models.ForeignKey
+    imported_via = models.ForeignKey('ImportQueue', blank=True, default=None, null=True)
+    parent = models.ForeignKey('self', default=None, blank=True, null=True)
     name = models.CharField(max_length=250, blank=False)
     schema = models.CharField(max_length=250, blank=True)
     database = models.CharField(max_length=250, blank=True, help_text="Leave blank to use default connection specified in Django's setings.py file")
     
-    CAT_CHOICES = [
+    STG_CAT_CHOICES = [
         ('CORE_SOURCE','CORE data in wide format, from source (no derived/updated cols)'),
         ('CHARGES_SOURCE','Charges data, from source (no wide-to-long conversion)'),
         ('DAYSTOEVENT_SOURCE','DaysToEvent and VisitLink data, from source (straight csv load)'),
+        ('AHALINK_SOURCE','American Hospital Association link data, from source'),
         ('CHARGES','Charges data in long format (converted from wide source if necessary)'),
-        ('DX','Diagnoses data in long format (extracted from CORE_SOURCE)'),
-        ('PR','Procedures data in long format (extracted from CORE_SOURCE)'),
-        ('UFLAGS','Utilization flag data in long format (generated from CHARGES and PR)'),
-        ('CORE_PROCESSED','CORE data in wide format plus wide data from CHARGES (if not originally present); DaysToEvent and VisitLink fields updated (if not originally present) and wide data from UFlags'),
+        ('DX','Diagnoses data in long format (from CORE_SOURCE)'),
+        ('PR','Procedures data in long format (from CORE_SOURCE)'),
+        ('UFLAGS','Utilization flag data, long format (from CORE_SOURCE, CHARGES, and PR)'),
+        ('CORE_PROCESSED','CORE Processed: CORE, DaysToEvent/VisitLink, wide charges, wide uflags'),
     ]
-    category = models.CharField(max_length=100, choices=CAT_CHOICES, blank=False)
+    category = models.CharField(max_length=100, choices=STG_CAT_CHOICES, blank=False)
+    wip = models.BooleanField(default=True, help_text="This StagingTable is work-in-progress, and should not be used.")
+    
     records = models.IntegerField()
     #TODO: consider adding a col count
     #col_count = models.IntegerField(blank=False)
@@ -471,7 +732,7 @@ class StagingTable(models.Model):
     #voided_by = models.ForeignKey(django user object) #unimplemented
     
     def __unicode__(self):
-        return "StagingTable %d (%s, %s)" % (self.pk, self.category, self.name)
+        return "StagingTable %d: %s" % (self.pk, self.name)
 
 
 class Column(models.Model):
@@ -479,7 +740,7 @@ class Column(models.Model):
     #... or if this is to know what columns are available from each imported state (*I think it's this one)
     #... or agnostic of import status, what columns are in a discovered loadfile
     #... or what
-    iq = models.ForeignKey('ImportQueue')
+    stg = models.ForeignKey('StagingTable', null=True)
     name = models.CharField(max_length=200, blank=False,
                             db_index=True)
     
