@@ -156,7 +156,7 @@ def source_detail(request, source_id):
     return render(request, template, context)
 
 
-def file_inventory(request):
+def file_inventory(request, msg_pass_through=[]):
     logger.debug('File inventory requested')
     known_files = File.objects.all()
     content_files = known_files.exclude(file_type__content_type__in=['LOAD'])
@@ -184,14 +184,14 @@ def file_inventory(request):
                     'display': 'Match discovered files',
                     'href': reverse('file_match'),
                     }],
+        'messages': msg_pass_through,
         }
     template = 'djhcup_staging_inventory.html'
     return render(request, template, context)
 
 
 def file_discover(request):
-    template = 'djhcup_staging_base.html'
-    
+    messages = []
     discovered = []
     sources = DataSource.objects.all()
     file_types = FileType.objects.all().prefetch_related('source')
@@ -271,18 +271,19 @@ def file_discover(request):
             
             result['hit_count'] = len(hits)
             result['discovered_count'] = len(discovered)
-            logger.info("%d file(s) already in the database. Added %d file(s) not previously in database" % (len(hits) - len(discovered), len(discovered)))
+            message = "%d file(s) already in the database. Added %d file(s) not previously in database" % (len(hits) - len(discovered), len(discovered))
         
     else:
-        logger.info("No patterns found")
-            
-    context = {
-        'title': 'File Discovery',
-        }
-    return render(request, template, context)
+        message = "No patterns found"
+    logger.info(message)
+    
+    messages.append(dict(level="info", content=message))
+    
+    return file_inventory(request, msg_pass_through=messages)
 
 
-def file_match(request):    
+def file_match(request):
+    messages = []
     #grab all known files and split them into content vs load files
     known_files = File.objects.all()
     unmatched_content = known_files.filter(loadfile__isnull=True).exclude(file_type__content_type__in=['LOAD'])
@@ -318,13 +319,12 @@ def file_match(request):
             unmatched.save()
             matches_made += 1
     
-    logger.info("Matched %s pairs of files with unambiguous similarity." % matches_made)
+    message = "Matched %s pairs of files with unambiguous similarity." % matches_made
+    logger.info(message)
     
-    context = {
-        'title': 'File Matching',
-        }
-    template = 'djhcup_staging_base.html' #automagically looks in the templates directory for this app
-    return render(request, template, context)
+    messages.append(dict(level="info", content=message))
+    
+    return file_inventory(request, msg_pass_through=messages)
 
 
 def src_gen_master_table(request, source_id):
@@ -337,7 +337,7 @@ def src_gen_master_table(request, source_id):
     cursor = cnxns[DJHCUP_DB_ENTRY].cursor()
 
     # need later for table creation, but want the value to be based on the start of the process
-    now = datetime.datetime.utcnow().replace(tzinfo=utc).strftime("%Y%m%d%H%M%S")    
+    now = datetime.datetime.utcnow().replace(tzinfo=utc).strftime("%Y%m%d%H%M%S")
     db_table = "%s_%s" % (source.mr_descriptor, now)
     pk_fields = []
     index_fields = []
@@ -465,8 +465,8 @@ def batch_detail(request, batch_id):
 
 
 def batch_import(request, batch_id):
-    """Schedules the indicated batch_id for import using Celery"""
-    logger.debug('Batch %s import requested' % batch_id)
+    """Schedules the indicated ImportBatch for import using Celery"""
+    logger.debug('ImportBatch %s requested' % batch_id)
     
     # import tasks for this module
     from djhcup_staging import tasks
@@ -485,6 +485,27 @@ def batch_import(request, batch_id):
     return render(request, template, context)
 
 
+def batch_stage(request, batch_id):
+    """Schedules the indicated StagingBatch for staging using Celery"""
+    logger.debug('StagingBatch %s requested' % batch_id)
+    
+    # import tasks for this module
+    from djhcup_staging import tasks
+    
+    logger.info('Attempting to call a Celery task for StagingBatch %i' % (int(batch_id)))
+    tresult = tasks.batch_stage.delay(batch_id)
+    
+    response = "StagingBatch has been dispatched to Celery in thread %s; its completion state is %s" % (tresult, tresult.state)
+    logger.info(response)
+    
+    context = {
+        'title': 'Import Batch',
+        'response_messages': [{'content': response, 'type': 'info'}],
+        }
+    template = 'djhcup_staging_base.html'
+    return render(request, template, context)
+
+
 def batch_reset_dead(request):
     """Dispatches a celery task to reset interrupted ImportBatch objects and their associated ImportQueue objects.
     """
@@ -502,4 +523,37 @@ def batch_reset_dead(request):
         }
     template = 'djhcup_staging_base.html'
     return render(request, template, context)
-    
\ No newline at end of file
+
+
+def shovel_all_core_source(request):
+    """Shovels all the core shit into a partprocessed table for now"""
+    # TODO: redo this entirely
+    logger.info('shovel_all import requested')
+    print "wtf mates"
+    
+    # import celery for group jobbing
+    import celery
+    logger.info('imported celery')
+    
+    # import tasks for this module
+    from djhcup_staging.tasks import pg_shovel_ab
+    logger.info('imported pg_shovel_ab task')
+    
+    core_st_qs = StagingTable.objects.filter(category='CORE_SOURCE')
+    logger.info('pulled core_st_qs with %s results' % len(core_st_qs))
+    
+    job = celery.group(
+        pg_shovel_ab.si(stg.pk) for stg in core_st_qs
+        )
+    result = job()
+    logger.info('triggered job with these results so far: %s' % result)
+    
+    response = "Group job has been dispatched to Celery"
+    logger.info(response)
+    
+    context = {
+        #'title': 'Import Batch',
+        #'response_messages': [{'content': response, 'type': 'info'}],
+        }
+    template = 'djhcup_staging_base.html'
+    return render(request, template, context)
\ No newline at end of file
